import 'dart:async';

import 'package:bea_flutter_package/bea_services/dtos/bea_models/base_models/error/bea_error.dart';
import 'package:bea_flutter_package/bea_services/dtos/bea_models/base_models/layout/get_layout_by_role_id_input.dart';
import 'package:bea_flutter_package/bea_services/dtos/bea_models/base_models/layout/get_layout_by_role_id_output.dart';
import 'package:bea_flutter_package/bea_services/dtos/flutter_models/input/generic_server_input.dart';
import 'package:bea_flutter_package/bea_services/dtos/flutter_models/output/bea_session.dart';
import 'package:bea_flutter_package/bea_services/utils/url_utils.dart';
import 'package:dio/dio.dart';
import 'package:f_logs/model/flog/flog.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';

import '../dtos/flutter_models/output/generic_server_response.dart';
import 'bea_service_manager.dart';

class BeaAppLayoutManager{
  final BeaServiceManager bsm;

  String _baseUrl = "";
  FlutterSecureStorage storage;

  BeaAppLayoutManager(this.bsm){
    try{
      if(bsm == null){
        throw new Exception("BeaServiceManager not initialized");
      }

      _baseUrl = bsm.getBaseUrl();
      storage = bsm.getBeaStorage();
    }on Exception catch (e){
      print('Initialization ${this.runtimeType.toString()} error');
      FLog.error(
          className: this.runtimeType.toString(),
          methodName: "Costructor",
          text: 'Initialization ${this.runtimeType.toString()} error - $e');
      throw e;
    }
  }

  Future<GenericServerResponse<GetLayoutByRoleIdOutput>> getLayoutByRoleId(GenericServerInput<GetLayoutByRoleIdInput> input) async {
    GenericServerResponse<GetLayoutByRoleIdOutput> genericServerResponse = new GenericServerResponse<GetLayoutByRoleIdOutput>();
    try{

      if(input.input == null){
        throw new Exception("Input not found.");
      }

      String url = UrlUtils.getUrl(_baseUrl, UrlUtils.URL_GET_LAYOUT_BY_ROLE_ID);

      String jsonInput = input.input.toJson().toString();

      Response response = await bsm.genericBeaHttpPost(url, jsonInput);
      int statusCode = response.statusCode;

      switch(statusCode){
        case 200:
          genericServerResponse.data = GetLayoutByRoleIdOutput.fromJson(response.data['result']);
          break;
        case 500:
          BeaError error = BeaError.fromJson(response.data['error']);
          if (error != null) {
            genericServerResponse.beaError = error;
          } else {
            genericServerResponse.beaError = new BeaError("Unknown Error", 0, "");
          }
          break;
        case 401:
          genericServerResponse.beaSession = new BeaSession(true);
          break;
      }
    }on Exception catch (e){
      print('Error to get BEA Layout by role id');
      FLog.error(
          className: this.runtimeType.toString(),
          methodName: "GetLayoutByRoleId",
          text: 'Error to get BEA Layout by role id - $e');
      throw e;
    }

    return genericServerResponse;
  }
}