import 'dart:async';
import 'dart:convert';

import 'package:bea_flutter_package/bea_services/dtos/bea_models/base_models/error/bea_error.dart';
import 'package:bea_flutter_package/bea_services/dtos/bea_models/base_models/roles/get_user_roles_async_input.dart';
import 'package:bea_flutter_package/bea_services/dtos/flutter_models/input/generic_server_input.dart';
import 'package:bea_flutter_package/bea_services/dtos/flutter_models/output/bea_session.dart';
import 'package:bea_flutter_package/bea_services/utils/url_utils.dart';
import 'package:dio/dio.dart';
import 'package:f_logs/model/flog/flog.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';

import '../dtos/flutter_models/output/generic_server_response.dart';
import 'bea_service_manager.dart';

class BeaRolesManager{
  final BeaServiceManager bsm;

  String _baseUrl = "";
  FlutterSecureStorage storage;

  BeaRolesManager(this.bsm){
    try{
      if(bsm == null){
        throw new Exception("BeaServiceManager not initialized");
      }

      _baseUrl = bsm.getBaseUrl();
      storage = bsm.getBeaStorage();
    }on Exception catch (e){
      print('Initialization ${this.runtimeType.toString()} error');
      FLog.error(
          className: this.runtimeType.toString(),
          methodName: "Costructor",
          text: 'Initialization ${this.runtimeType.toString()} error - $e');
      throw e;
    }
  }

  Future<GenericServerResponse<List<String>>> GetUserRolesAsync(GenericServerInput<GetUserRolesAsyncInput> input) async {
    GenericServerResponse<List<String>> genericServerResponse = new GenericServerResponse<List<String>>();
    try{

      if(input.input == null){
        throw new Exception("Input not found.");
      }

      String url = UrlUtils.getUrl(_baseUrl, UrlUtils.URL_GET_USER_ROLE);

      String jsonInput = input.input.toJson().toString();

      Response response = await bsm.genericBeaHttpPost(url, jsonInput);
      int statusCode = response.statusCode;

      switch(statusCode){
        case 200:
          genericServerResponse.data = (response.data['result'] as List<dynamic>).cast<String>();
          break;
        case 500:
          BeaError error = BeaError.fromJson(response.data['error']);
          if (error != null) {
            genericServerResponse.beaError = error;
          } else {
            genericServerResponse.beaError = new BeaError("Unknown Error", 0, "");
          }
          break;
        case 401:
          genericServerResponse.beaSession = new BeaSession(true);
          break;
      }
    }on Exception catch (e){
      print('Error to get BEA roles');
      FLog.error(
          className: this.runtimeType.toString(),
          methodName: "GetUserRolesAsync",
          text: 'Error to get BEA roles - $e');
      throw e;
    }

    return genericServerResponse;
  }
}